// () ..,2003
// http://alwd.narod.ru
// mailto:alwd@narod.ru

unit Sights;

interface

uses
  Windows, Graphics, ExtCtrls;

type

  // TSight - "",    Image  .
  //      :
  //
  //  Create
  //    ...
  //  ShowFirst
  //  ShowNext
  //  Hide
  //    ...
  //  Show*  Show*XY - .     .

  TSightMode = (smNone,smSight,smHorizontal,smVertical,smBoth);

  TSight = class
    private
      fPoint: TPoint;
      fImage: TImage;
      fMode: TSightMode;
      fShowed: boolean;
      fOldPen: TPen;
      procedure Show;
      procedure SetImage(Image: TImage);
    public
      constructor Create;
      destructor Destroy; override;
      procedure ShowFirst(Image: TImage; Point: TPoint; Mode: TSightMode);
      procedure ShowFirstXY(Image: TImage; X,Y: integer; Mode: TSightMode);
      procedure ShowNext(Point: TPoint);
      procedure ShowNextXY(X,Y: integer);
      procedure Repaint;
      procedure Hide;
      function  ColorUnderCursor: TColor;
      property  Visible: boolean read fShowed;
      property Image: TImage read fImage write SetImage;
  end;


// TCanvasHint -  hint  Canvas,    
//   .
//     TSight.
//
//  :
//  Gap -        
//  Transparent -  TRUE,  hint  
//  AlwaysInWindow -  TRUE,  hint     
//
//  : (Gap = 0), (Transparent = FALSE), (AlwaysInWindow = TRUE)

  TCanvasHint = class
    private
      fImage: TImage;
      fRect: TRect;
      fFont: TFont;
      fTextColor: TColor;
      fForeColor: TColor;
      fOldFont: TFont;
      fBitmap: TBitmap;
      fShowed: boolean;
      fHint: string;
      fXGap: integer;
      fYGap: integer;
      fTransparent: boolean;
      fAlwaysInWindow: boolean;
      procedure DoShow;
      procedure DoHide;
      procedure SetFont(Font: TFont);
      procedure SetXGap(XGap: integer);
      procedure SetYGap(YGap: integer);
      procedure SetTextColor(TextColor: TColor);
      procedure SetForeColor(ForeColor: TColor);
      procedure SetTransparent(Transparent: boolean);
      procedure SetAlwaysInWindow(AlwaysInWindow: boolean);
      procedure SetImage(Image: TImage);
    public
      constructor Create;
      destructor Destroy; override;
      procedure ShowFirst(Image: TImage; Point: TPoint; const Hint: string);
      procedure ShowFirstXY(Image: TImage; X,Y: integer; const Hint: string);
      procedure ShowNext(Point: TPoint; const Hint: string);
      procedure ShowNextXY(X,Y: integer; const Hint: string);
      procedure Repaint;
      procedure Hide;
      property Visible: boolean read fShowed;
      property Font: TFont read fFont write SetFont;
      property TextColor: TColor read fTextColor write SetTextColor;
      property ForeColor: TColor read fForeColor write SetForeColor;
      property XGap: integer read fXGap write SetXGap;
      property YGap: integer read fYGap write SetYGap;
      property Transparent: boolean read fTransparent write SetTransparent;
      property AlwaysInWindow: boolean read fAlwaysInWindow write SetAlwaysInWindow;
      function HintWidth(Hint: string): integer;
      function HintHeight(Hint: string): integer;
      property Image: TImage read fImage write SetImage;
  end;


// THintSight - ,     Canvas   .
//      ,     
//     ,   .

  THintSight = class(TSight)
    public
      Hint: TCanvasHint;
      constructor Create;
      destructor Destroy; override;
      procedure ShowFirst(Image: TImage; Point: TPoint; Mode: TSightMode; AHint: string);
      procedure ShowFirstXY(Image: TImage; X,Y: integer; Mode: TSightMode; AHint: string);
      procedure ShowNext(Point: TPoint; AHint: string);
      procedure ShowNextXY(X,Y: integer; AHint: string);
  end;

//    
procedure ShowCross(Image: TImage; Size: integer; Color: TColor; Point: TPoint);
procedure ShowCrossXY(Image: TImage; Size: integer; Color: TColor; X,Y: integer);

implementation

(*==================================================================*)
(*==================== Sight ======================================*)
(*==================================================================*)

constructor TSight.Create;
begin
  inherited Create;
  fShowed := FALSE;
  fOldPen := TPen.Create;
end;


destructor TSight.Destroy;
begin
  fOldPen.Free;
  inherited Destroy;
end;


procedure TSight.SetImage;
begin
  if Assigned(Image) then fImage := Image;
end;


procedure TSight.Show;
begin
  with fImage.Canvas do with fPoint do begin
    fOldPen.Assign(Pen);
    Pen.Color := clWhite;
    Pen.Mode := pmXor;
    Pen.Style := psSolid;
    Pen.Width := 1;
    case fMode of
      smHorizontal:
        begin
          MoveTo(0,Y);
          LineTo(fImage.Width,Y);
        end;
      smVertical:
        begin
          MoveTo(X,0);
          LineTo(X,fImage.Height);
        end;
      smBoth:
        begin
          MoveTo(0,Y);
          LineTo(fImage.Width,Y);
          MoveTo(X,0);
          LineTo(X,fImage.Height);
        end;
      smSight:
        begin
          MoveTo(0,Y); LineTo(fImage.Width,Y);
          MoveTo(X,0); LineTo(X,fImage.Height);
          MoveTo(X-5,Y-5); LineTo(X-2,Y-5);
          MoveTo(X-5,Y-5); LineTo(X-5,Y-2);
          MoveTo(X-5,Y+5); LineTo(X-2,Y+5);
          MoveTo(X-5,Y+5); LineTo(X-5,Y+2);
          MoveTo(X+5,Y-5); LineTo(X+2,Y-5);
          MoveTo(X+5,Y-5); LineTo(X+5,Y-2);
          MoveTo(X+5,Y+5); LineTo(X+2,Y+5);
          MoveTo(X+5,Y+5); LineTo(X+5,Y+2);
        end;
    end;
    Pen.Assign(fOldPen);
  end;
end;


procedure TSight.ShowFirst;
begin
  ShowFirstXY(Image,Point.X,Point.Y,Mode);
end;


procedure TSight.ShowFirstXY;
begin
  fShowed := TRUE;
  fImage := Image;
  fPoint.X := X;
  fPoint.Y := Y;
  fMode := Mode;
  Show;
end;


procedure TSight.Repaint;
begin
  fShowed := TRUE;
  Show;
end;


procedure TSight.ShowNext;
begin
  ShowNextXY(Point.X,Point.Y);
end;


procedure TSight.ShowNextXY;
begin
  if fShowed then begin
    Show;
    fPoint.X := X;
    fPoint.Y := Y;
    Show;
  end;
end;


procedure TSight.Hide;
begin
  if fShowed then begin
    Show;
    fShowed := FALSE;
  end;
end;


function TSight.ColorUnderCursor;
begin
  if fShowed then Result := fImage.Canvas.Pixels[fPoint.X,fPoint.Y]
  else Result := 0;
end;

(*==================================================================*)
(*================ TCanvasHint =====================================*)
(*==================================================================*)

constructor TCanvasHint.Create;
begin
  inherited Create;
  fShowed := FALSE;
  fOldFont := TFont.Create;
  fFont := TFont.Create;
  fBitmap := TBitmap.Create;
  fTextColor := clBlack;
  fForeColor := clYellow;
  fTransparent := FALSE;
  fAlwaysInWindow := TRUE;
  fXGap := 0;
  fYGap := 0;
end;


destructor TCanvasHint.Destroy;
begin
  fBitmap.Free;
  fFont.Free;
  fOldFont.Free;
  inherited Destroy;
end;


procedure TCanvasHint.SetImage;
begin
  if Assigned(Image) then fImage := Image;
end;


procedure TCanvasHint.DoShow;
var
  d: integer;
begin
  fOldFont.Assign(fImage.Canvas.Font);
  fImage.Canvas.Font.Assign(fFont);
  fBitmap.Canvas.Brush.Style := fImage.Canvas.Brush.Style;
  fBitmap.Canvas.Brush.Color := fImage.Canvas.Brush.Color;
  if fTransparent then begin
    fImage.Canvas.Brush.Style := bsClear;
    end
  else begin
    fImage.Canvas.Brush.Style := bsSolid;
    fImage.Canvas.Brush.Color := fForeColor;
  end;
  fImage.Canvas.Font.Color := fTextColor;
  fRect.Right := fRect.Left + fImage.Canvas.TextWidth(fHint);
  fRect.Bottom := fRect.Top + fImage.Canvas.TextHeight(fHint);
  if fAlwaysInWindow then begin
    if fRect.Right > fImage.Width then begin
      d := fRect.Right - fRect.Left + fXGap + fXGap;
      Dec(fRect.Left,d);
      Dec(fRect.Right,d);
    end;
    if fRect.Bottom > fImage.Height then begin
      d := fRect.Bottom - fRect.Top + fYGap + fYGap;
      Dec(fRect.Top,d);
      Dec(fRect.Bottom,d);
    end;
    if fRect.Left < 0 then begin
      fRect.Right := fRect.Right - fRect.Left;
      fRect.Left := 0;
    end;
    if fRect.Top < 0 then begin
      fRect.Bottom := fRect.Bottom - fRect.Top;
      frect.Top := 0;
    end;
  end;
  fBitmap.Canvas.CopyRect(fRect,fImage.Canvas,fRect);
  fImage.Canvas.TextOut(fRect.Left,fRect.Top,fHint);
  fImage.Canvas.Brush.Style := fBitmap.Canvas.Brush.Style;
  fImage.Canvas.Brush.Color := fBitmap.Canvas.Brush.Color;
  fImage.Canvas.Font.Assign(fOldFont);
end;


procedure TCanvasHint.DoHide;
begin
  fImage.Canvas.CopyRect(fRect,fBitmap.Canvas,fRect);
end;


procedure TCanvasHint.ShowFirst;
begin
  ShowFirstXY(Image,Point.X,Point.Y,Hint);
end;


procedure TCanvasHint.ShowFirstXY;
begin
  fShowed := TRUE;
  fImage := Image;
  fBitmap.Width := fImage.Width;
  fBitmap.Height := fImage.Height;
  fRect.Left := X + fXGap;
  fRect.Top := Y + fYGap;
  fHint := Hint;
  DoShow;
end;


procedure TCanvasHint.Repaint;
begin
  fShowed := TRUE;
  DoShow;
end;


procedure TCanvasHint.ShowNext;
begin
  ShowNextXY(Point.X,Point.Y,Hint);
end;


procedure TCanvasHint.ShowNextXY;
begin
  if fShowed then begin
    DoHide;
    fRect.Left := X + fXGap;
    fRect.Top := Y + fYGap;
    fHint := Hint;
    DoShow;
  end;
end;


procedure TCanvasHint.Hide;
begin
  if fShowed then begin
    DoHide;
    fShowed := FALSE;
  end;
end;


procedure TCanvasHint.SetFont;
begin
  if fShowed then begin
    DoHide;
    fFont.Assign(Font);
    DoShow;
    end
  else fFont.Assign(Font);
end;


procedure TCanvasHint.SetXGap;
begin
  if fXGap = XGap then Exit;
  if fShowed then begin
    DoHide;
    fXGap := XGap;
    DoShow;
    end
  else fXGap := XGap;
end;


procedure TCanvasHint.SetYGap;
begin
  if fYGap = YGap then Exit;
  if fShowed then begin
    DoHide;
    fYGap := YGap;
    DoShow;
    end
  else fYGap := YGap;
end;


procedure TCanvasHint.SetTextColor;
begin
  if fTextColor = TextColor then Exit;
  if fShowed then begin
    DoHide;
    fTextColor := TextColor;
    DoShow;
    end
  else fTextColor := TextColor;
end;


procedure TCanvasHint.SetForeColor;
begin
  if fForeColor = ForeColor then Exit;
  if fShowed then begin
    DoHide;
    fForeColor := ForeColor;
    DoShow;
    end
  else fForeColor := ForeColor;
end;


procedure TCanvasHint.SetTransparent;
begin
  if fTransparent = Transparent then Exit;
  if fShowed then begin
    DoHide;
    fTransparent := Transparent;
    DoShow;
    end
  else fTransparent := Transparent;
end;


procedure TCanvasHint.SetAlwaysInWindow;
begin
  if fAlwaysInWindow = AlwaysInWindow then Exit;
  if fShowed then begin
    DoHide;
    fAlwaysInWindow := AlwaysInWindow;
    DoShow;
    end
  else fAlwaysInWindow := AlwaysInWindow;
end;


function TCanvasHint.HintWidth;
begin
  Result := fImage.Canvas.TextWidth(Hint);
end;


function TCanvasHint.HintHeight;
begin
  Result := fImage.Canvas.TextHeight(Hint);
end;

(*==================================================================*)
(*================== THintSight ====================================*)
(*==================================================================*)

constructor THintSight.Create;
begin
  inherited Create;
  Hint := TCanvasHint.Create;
end;


destructor THintSight.Destroy;
begin
  Hint.Free;
  inherited Destroy;
end;


procedure THintSight.ShowFirst;
begin
  ShowFirstXY(Image,Point.X,Point.Y,Mode,AHint);
end;


procedure THintSight.ShowFirstXY;
begin
  fShowed := TRUE;
  fImage := Image;
  fPoint.X := X;
  fPoint.Y := Y;
  fMode := Mode;
  Hint.ShowFirstXY(fImage,X,Y,AHint);
  Show;
end;


procedure THintSight.ShowNext;
begin
  ShowNextXY(Point.X,Point.Y,AHint);
end;


procedure THintSight.ShowNextXY;
begin
  if fShowed then begin
    Show;
    fPoint.X := X;
    fPoint.Y := Y;
    Hint.ShowNextXY(X,Y,AHint);
    Show;
  end;
end;

(*==================================================================*)
(*==================  =====================================*)
(*==================================================================*)

procedure ShowCrossXY;
var
  Pen: TPen;
begin
  Pen := TPen.Create;
  Pen.Assign(Image.Canvas.Pen);
  with Image.Canvas do begin
    Pen.Color := Color;
    Pen.Style := psSolid;
    Pen.Mode := pmCopy;
    Pen.Width := 1;
    Size := Size shr 1;
    MoveTo(X-Size,Y); LineTo(X+Size,Y);
    MoveTo(X,Y-Size); LineTo(X,Y+Size);
  end;
  Image.Canvas.Pen.Assign(Pen);
  Pen.Free;
end;


procedure ShowCross;
begin
  ShowCrossXY(Image,Size,Color,Point.X,Point.Y);
end;

end.

